/***	******************************************************************************
 * Dragomir Andreea-Alisa
 * 332CC
 * Joc "X si 0"
 *********************************************************************************/

#ifndef __PROIECT_H__
#define __PROIECT_H__

#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <avr/sleep.h>
#include <stdlib.h>
#include <stdint.h>

typedef struct _led {
	uint8_t pos[2];
	volatile uint8_t *port[2];
} led;

#define EMPTY	2
#define RED		0
#define GREEN	1

#define UP		0
#define DOWN	1
#define LEFT	2
#define RIGHT	3

led map[3][3];
uint8_t board[3][3];

#endif

int currX = 0, currY = 0;

/**
Initializare porturi de iesire(ledurile)
dar si de intrare(butoanele)
*/
void initPorts()
{

	DDRD = (1<<PD0) | (1<<PD1) | (1<<PD2) | (1<<PD3) | (1<<PD4);
	DDRD |= (1<<PD6);
	DDRD |= (1<<PD5);
	DDRC = 0xFF;
	DDRA = 0xFF;
	PORTA = 0;
	PORTC = 0;
	PORTD = 0x1F;
}

/**
Inchidere porturi de iesire
*/
void all_out() {
	PORTD &= (~(1<<PD5));
	PORTD &= (~(1<<PD6));
	PORTC = 0;
	PORTA = 0;
}


/**
Aprinderea tuturor ledurilor
*/
void lightAll(int color) {
	int i, j;

	for (i = 0; i < 3; i++)
		for (j = 0; j < 3; j++)
			*(map[i][j].port[color]) |= (1 << map[i][j].pos[color]);
}


/**
Stingerea tuturor ledurilor
*/
void outAll(int color){
	int i, j;
	for (i = 0; i < 3; i++){
		for (j = 0; j < 3; j++){
			*(map[i][j].port[RED]) &= ~(1 << map[i][j].pos[RED]);
			*(map[i][j].port[GREEN]) &= ~(1 << map[i][j].pos[GREEN]);
		}
	}
}

/**
Apasarea butonului
*/
int button_pressed(int which) {
	if (bit_is_clear(PIND, which)) {
		_delay_ms(20);
		if (bit_is_clear(PIND,which)) return 1;
	}
	return 0;
}

/**
Cautarea urmatorului led liber, dupa apasarea tastei Enter
*/
void nextFreeLed(int *currX, int *currY) {
	int i, j;

	for(i = 0; i < 3; i++)
		for (j = 0; j < 3; j++)
			if (board[i][j] == EMPTY) {
				*currX = i;
				*currY = j;
				return;
			}
}

/**
Eliberare memorie matrice
*/
void cleanBoard(){
	int i, j;
	for (i = 0; i < 3; i++)
		for (j = 0; j < 3; j++)
			board[i][j] = EMPTY;
}


/**
Gaseste pozitie in functie de butonul apasat
*/
void find(int pos, int *currX, int *currY, int playerMove) {
	int i;
	int not_playerMove;
	not_playerMove = 1 - playerMove;
	switch(pos) {
		case RIGHT:
			for (i = 0; i< 3; i++) {
				(*currX)++;
				if (*currX > 2)
					*currX = 0;
				if (board[*currX][*currY] == EMPTY)
					break;
				else if(board[*currX][*currY] == not_playerMove
						&& (*currX) + 1 == 2)
				{
					*currX = 2;
				}
					
			}
			break;
			
		case LEFT:
			for (i = 0; i< 3; i++) {
				(*currX)--;
				if (*currX < 0)
					*currX = 2;
				if (board[*currX][*currY] == EMPTY)
					break;
				else if(board[*currX][*currY] == not_playerMove
						&& (*currX) - 1 == 0)
				{
					*currX = 0;
				}
			}
			break;
			
		case UP:
			for (i = 0; i< 3; i++) {
				(*currY)--;
				if (*currY < 0)
					*currY = 2;
				if (board[*currX][*currY] == EMPTY)
					break;
				else if(board[*currX][*currY] == not_playerMove
						&& (*currY) - 1 == 0)
				{
					*currY = 0;
				}
			}
			break;

		case DOWN:
			for (i = 0; i< 3; i++) {
				(*currY)++;
				if (*currY > 2)
					*currY = 0;
				if (board[*currX][*currY] == EMPTY)
					break;
				else if(board[*currX][*currY] == not_playerMove
						&& (*currY) + 1 == 2)
				{
					*currY = 2;
				}
			}
			break;
	}
}

/**
Functie executata la fiecare apasare de buton
*/
void executeFunction(int playerMove, int x){

	*(map[currX][currY].port[playerMove]) &= ~(1 << map[currX][currY].pos[playerMove]);
	
	if(x == 0)
		find(UP, &currX, &currY, playerMove);
	else if(x == 1)
		find(LEFT, &currX, &currY, playerMove);
	else if(x == 2)
		find(RIGHT, &currX, &currY, playerMove);
	else if(x == 3)
		find(DOWN, &currX, &currY, playerMove);
		
	*(map[currX][currY].port[playerMove]) |= (1 << map[currX][currY].pos[playerMove]);
	
	_delay_ms(200);
	

}

/**
Testare placuta daca am ajuns la final
*/
int testBoard(int color, int currX, int currY) {

	if(board[currX][0] == color && 
		board[currX][1] ==  color && board[currX][2] == color)
	{
		return 1;
	}
	
	else if(board[0][currY] == color && 
		board[1][currY] ==  color && board[2][currY] == color)
	{
		return 1;
	}
	
	else if(board[0][0] == color && 
		board[1][1] ==  color && board[2][2] == color)
	{
		return 1;
	}
	
	else if(board[0][2] == color && 
		board[1][1] ==  color && board[2][0] == color)
	{
		return 1;
	}

	return 0;
}

/**
Functia play
*/
void play() {
	int playerMove = RED, moves = 0;

	while (1) {
		if (button_pressed(PD0)) {
			executeFunction(playerMove, 0);
			continue;
		}

		if (button_pressed(PD3)) {
			executeFunction(playerMove, 1);
			continue;
		}

		if (button_pressed(PD2)) {
		
			*(map[currX][currY].port[playerMove]) |= (1 << map[currX][currY].pos[playerMove]);
			board[currX][currY] = playerMove;
			moves++;
			if (moves >= 5 && testBoard(playerMove,currX, currY)) {
//				outAll();
				lightAll(playerMove);
				return;
			}

			if (moves == 9)
				return;

			nextFreeLed(&currX, &currY);
			playerMove = 1 - playerMove;
		}

		if (button_pressed(PD1)) {
			executeFunction(playerMove, 2);
			continue;
		}
		if (button_pressed(PD4)) {
			executeFunction(playerMove, 3);
			continue;
		}
		
		*(map[currX][currY].port[playerMove]) ^= (1 << map[currX][currY].pos[playerMove]);
		_delay_ms(200);
	}
}

/**
Setare matrice in functie de cum sunt conectati
pinii intre ei
*/
void setMap() {
	map[0][0].pos[RED] = PC5;
	map[0][0].port[RED] = &PORTC;
	map[0][0].pos[GREEN] = PC4;
	map[0][0].port[GREEN] = &PORTC;

	map[0][1].pos[RED] = PC7;
	map[0][1].port[RED] = &PORTC;
	map[0][1].pos[GREEN] = PD6;
	map[0][1].port[GREEN] = &PORTD;

	map[0][2].pos[RED] = PA0;
	map[0][2].port[RED] = &PORTA;
	map[0][2].pos[GREEN] = PA1;
	map[0][2].port[GREEN] = &PORTA;

	map[1][0].pos[RED] = PC3;
	map[1][0].port[RED] = &PORTC;
	map[1][0].pos[GREEN] = PC2;
	map[1][0].port[GREEN] = &PORTC;

	map[1][1].pos[RED] = PC6;
	map[1][1].port[RED] = &PORTC;
	map[1][1].pos[GREEN] = PA6;
	map[1][1].port[GREEN] = &PORTA;

	map[1][2].pos[RED] = PA2;
	map[1][2].port[RED] = &PORTA;
	map[1][2].pos[GREEN] = PA3;
	map[1][2].port[GREEN] = &PORTA;

	map[2][0].pos[RED] = PC1;
	map[2][0].port[RED] = &PORTC;
	map[2][0].pos[GREEN] = PC0;
	map[2][0].port[GREEN] = &PORTC;

	map[2][1].pos[RED] = PD5;
	map[2][1].port[RED] = &PORTD;
	map[2][1].pos[GREEN] = PA7;
	map[2][1].port[GREEN] = &PORTA;

	map[2][2].pos[RED] = PA4;
	map[2][2].port[RED] = &PORTA;
	map[2][2].pos[GREEN] = PA5;
	map[2][2].port[GREEN] = &PORTA;
}

int main()
{
	
	initPorts();
	setMap();

	while(1) {
		cleanBoard();
		play();
		while (!button_pressed(PD2));
		all_out();
		_delay_ms(300);
	}
	
	return 0;
}
